@@ -6,6 +6,7 @@ from pathlib import Path
 from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
 
 from pip._vendor.packaging.markers import InvalidMarker
+from pip._vendor.packaging.requirements import InvalidRequirement
 from pip._vendor.pkg_resources import Requirement as PackageRequirement
 from pip._vendor.pkg_resources import RequirementParseError, safe_name
 
@@ -211,10 +212,7 @@ class Requirement:
         )
 
     def as_ireq(self, **kwargs) -> InstallRequirement:
-        if self.is_file_or_url:
-            line_for_req = self.as_line(True)
-        else:
-            line_for_req = self.as_line()
+        line_for_req = self.as_line()
         if self.editable:
             line_for_req = line_for_req[3:].strip()
             ireq = install_req_from_editable(line_for_req, **kwargs)
@@ -258,7 +256,11 @@ class FileRequirement(Requirement):
         if not self.path:
             return None
         result = self.path.as_posix()
-        if not self.path.is_absolute():
+        if (
+            not self.path.is_absolute()
+            and not result.startswith("./")
+            and not result.startswith("../")
+        ):
             result = "./" + result
         return result
 
@@ -311,20 +313,15 @@ class FileRequirement(Requirement):
     def key(self) -> Optional[str]:
         return self.project_name.lower() if self.project_name else None
 
-    def as_line(self, for_ireq: bool = False) -> str:
-        editable = "-e " if self.editable else ""
+    def as_line(self) -> str:
         project_name = f"{self.project_name}" if self.project_name else ""
         extras = f"[{','.join(sorted(self.extras))}]" if self.extras else ""
-        if self.path and not for_ireq and self.editable and not project_name:
-            location = self.str_path
-        else:
-            location = self.url
         marker = self._format_marker()
-        if for_ireq and project_name:
-            return f"{editable}{location}#egg={project_name}{extras}{marker}"
-        else:
-            delimiter = " @ " if project_name else ""
-            return f"{editable}{project_name}{extras}{delimiter}{location}{marker}"
+        url = url_without_fragments(self.url)
+        if self.editable:
+            return f"-e {url}#egg={project_name}{extras}{marker}"
+        delimiter = " @ " if project_name else ""
+        return f"{project_name}{extras}{delimiter}{url}{marker}"
 
     def _parse_name_from_url(self) -> None:
         parsed = urlparse.urlparse(self.url)
@@ -346,7 +343,6 @@ class FileRequirement(Requirement):
         ):
             raise RequirementError(f"The local path '{self.path}' is not installable.")
         result = SetupReader.read_from_directory(self.path.absolute().as_posix())
-        # TODO: Assign a temp name for PEP 517 local package.
         self.name = result["name"]
 
 
@@ -381,8 +377,15 @@ class VcsRequirement(FileRequirement):
         return r
 
     def as_line(self) -> str:
-        editable = "-e " if self.editable else ""
-        return f"{editable}{self.vcs}+{self.url}{self._format_marker()}"
+        if self.editable:
+            return f"-e {self.vcs}+{self.url}{self._format_marker()}"
+        else:
+            extras = f"[{','.join(self.extras)}]" if self.extras else ""
+            ref = f"@{self.ref}" if self.ref else ""
+            return (
+                f"{self.project_name}{extras} @ {self.vcs}+{self.repo}"
+                f"{ref}{self._format_marker()}"
+            )
 
     def _parse_url(self) -> None:
         if self.url.startswith("git@"):
@@ -445,7 +448,7 @@ def parse_requirement(line: str, editable: bool = False) -> Requirement:
     else:
         try:
             r = NamedRequirement.parse(line)  # type: Requirement
-        except RequirementParseError as e:
+        except (RequirementParseError, InvalidRequirement) as e:
             m = FILE_REQ.match(line)
             if m is not None:
                 r = FileRequirement.parse(line, m.groupdict())
@@ -453,7 +456,9 @@ def parse_requirement(line: str, editable: bool = False) -> Requirement:
                 raise RequirementError(str(e)) from None
         else:
             if r.url:
-                r = FileRequirement(name=r.name, url=r.url, extras=r.extras)
+                r = FileRequirement(
+                    name=r.name, url=r.url, extras=r.extras, marker=r.marker
+                )
 
     if editable:
         if r.is_vcs or r.is_file_or_url and r.is_local_dir:
